@@ -153,6 +153,26 @@ struct message_t {
 	 */
 	bool (*get_request) (message_t *this);
 
+	/**
+	 * Set the version flag in the IKE header.
+	 */
+	void (*set_version_flag)(message_t *this);
+
+	/**
+	 * Get a reserved bit in the IKE header.
+	 *
+	 * @param nr			reserved bit to get in IKE header, 0-4
+	 * @return				TRUE if bit is set
+	 */
+	bool (*get_reserved_header_bit)(message_t *this, u_int nr);
+
+	/**
+	 * Set a reserved bit in the IKE header.
+	 *
+	 * @param nr			reserved bit to set in IKE header, 0-4
+	 */
+	void (*set_reserved_header_bit)(message_t *this, u_int nr);
+
 	/**
 	 * Append a payload to the message.
 	 *
@@ -180,6 +200,11 @@ struct message_t {
 	void (*add_notify) (message_t *this, bool flush, notify_type_t type,
 						chunk_t data);
 
+	/**
+	 * Disable automatic payload sorting for this message.
+	 */
+	void (*disable_sort)(message_t *this);
+
 	/**
 	 * Parses header of message.
 	 *
@@ -206,8 +231,6 @@ struct message_t {
 	 * @param aead		aead transform to verify/decrypt message
 	 * @return
 	 * 					- SUCCESS if parsing successful
-	 * 					- NOT_SUPPORTED if ciritcal unknown payloads found
-	 * 					- NOT_SUPPORTED if message type is not supported!
 	 *					- PARSE_ERROR if message parsing failed
 	 * 					- VERIFY_ERROR if message verification failed (bad syntax)
 	 * 					- FAILED if integrity check failed
@@ -234,6 +257,13 @@ struct message_t {
 	 */
 	status_t (*generate) (message_t *this, aead_t *aead, packet_t **packet);
 
+	/**
+	 * Check if the message has already been encoded using generate().
+	 *
+	 * @return			TRUE if message has been encoded
+	 */
+	bool (*is_encoded)(message_t *this);
+
 	/**
 	 * Gets the source host informations.
 	 *
@@ -281,6 +311,13 @@ struct message_t {
 	 */
 	enumerator_t * (*create_payload_enumerator) (message_t *this);
 
+	/**
+	 * Remove the payload at the current enumerator position.
+	 *
+	 * @param enumerator	enumerator created by create_payload_enumerator()
+	 */
+	void (*remove_payload_at)(message_t *this, enumerator_t *enumerator);
+
 	/**
 	 * Find a payload of a specific type.
 	 *
