@@ -111,11 +111,11 @@ struct proposal_substructure_t {
 	proposal_t * (*get_proposal) (proposal_substructure_t *this);
 
 	/**
-	 * Clones an proposal_substructure_t object.
+	 * Create an enumerator over transform substructures.
 	 *
-	 * @return		cloned object
+	 * @return			enumerator over transform_substructure_t
 	 */
-	proposal_substructure_t* (*clone) (proposal_substructure_t *this);
+	enumerator_t* (*create_substructure_enumerator)(proposal_substructure_t *this);
 
 	/**
 	 * Destroys an proposal_substructure_t object.
