@@ -53,6 +53,11 @@ struct private_eap_aka_peer_t {
 	 */
 	identification_t *reauth;
 
+	/**
+	 * EAP message identifier
+	 */
+	u_int8_t identifier;
+
 	/**
 	 * MSK
 	 */
@@ -72,8 +77,7 @@ struct private_eap_aka_peer_t {
 /**
  * Create a AKA_CLIENT_ERROR: "Unable to process"
  */
-static eap_payload_t* create_client_error(private_eap_aka_peer_t *this,
-										  u_int8_t identifier)
+static eap_payload_t* create_client_error(private_eap_aka_peer_t *this)
 {
 	simaka_message_t *message;
 	eap_payload_t *out;
@@ -82,7 +86,7 @@ static eap_payload_t* create_client_error(private_eap_aka_peer_t *this,
 	DBG1(DBG_IKE, "sending client error '%N'",
 		 simaka_client_error_names, AKA_UNABLE_TO_PROCESS);
 
-	message = simaka_message_create(FALSE, identifier, EAP_AKA,
+	message = simaka_message_create(FALSE, this->identifier, EAP_AKA,
 									AKA_CLIENT_ERROR, this->crypto);
 	encoded = htons(AKA_UNABLE_TO_PROCESS);
 	message->add_attribute(message, AT_CLIENT_ERROR_CODE,
@@ -124,7 +128,7 @@ static status_t process_identity(private_eap_aka_peer_t *this,
 			default:
 				if (!simaka_attribute_skippable(type))
 				{
-					*out = create_client_error(this, in->get_identifier(in));
+					*out = create_client_error(this);
 					enumerator->destroy(enumerator);
 					return NEED_MORE;
 				}
@@ -159,7 +163,7 @@ static status_t process_identity(private_eap_aka_peer_t *this,
 		default:
 			break;
 	}
-	message = simaka_message_create(FALSE, in->get_identifier(in), EAP_AKA,
+	message = simaka_message_create(FALSE, this->identifier, EAP_AKA,
 									AKA_IDENTITY, this->crypto);
 	if (id.len)
 	{
@@ -200,7 +204,7 @@ static status_t process_challenge(private_eap_aka_peer_t *this,
 			default:
 				if (!simaka_attribute_skippable(type))
 				{
-					*out = create_client_error(this, in->get_identifier(in));
+					*out = create_client_error(this);
 					enumerator->destroy(enumerator);
 					return NEED_MORE;
 				}
@@ -212,7 +216,7 @@ static status_t process_challenge(private_eap_aka_peer_t *this,
 	if (!rand.len || !autn.len)
 	{
 		DBG1(DBG_IKE, "received invalid EAP-AKA challenge message");
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 
@@ -258,7 +262,7 @@ static status_t process_challenge(private_eap_aka_peer_t *this,
 	 * reading encrypted attributes */
 	if (!in->verify(in, chunk_empty) || !in->parse(in))
 	{
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 
@@ -285,7 +289,7 @@ static status_t process_challenge(private_eap_aka_peer_t *this,
 	}
 	enumerator->destroy(enumerator);
 
-	message = simaka_message_create(FALSE, in->get_identifier(in), EAP_AKA,
+	message = simaka_message_create(FALSE, this->identifier, EAP_AKA,
 									AKA_CHALLENGE, this->crypto);
 	message->add_attribute(message, AT_RES, chunk_create(res, res_len));
 	*out = message->generate(message, chunk_empty);
@@ -320,7 +324,7 @@ static status_t process_reauthentication(private_eap_aka_peer_t *this,
 	{
 		DBG1(DBG_IKE, "received %N, but not expected",
 			 simaka_subtype_names, AKA_REAUTHENTICATION);
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 
@@ -330,7 +334,7 @@ static status_t process_reauthentication(private_eap_aka_peer_t *this,
 	/* verify MAC and parse again with decryption key */
 	if (!in->verify(in, chunk_empty) || !in->parse(in))
 	{
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 
@@ -351,7 +355,7 @@ static status_t process_reauthentication(private_eap_aka_peer_t *this,
 			default:
 				if (!simaka_attribute_skippable(type))
 				{
-					*out = create_client_error(this, in->get_identifier(in));
+					*out = create_client_error(this);
 					enumerator->destroy(enumerator);
 					return NEED_MORE;
 				}
@@ -363,7 +367,7 @@ static status_t process_reauthentication(private_eap_aka_peer_t *this,
 	if (!nonce.len || !counter.len)
 	{
 		DBG1(DBG_IKE, "EAP-AKA/Request/Reauthentication message incomplete");
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 
@@ -440,38 +444,38 @@ static status_t process_notification(private_eap_aka_peer_t *this,
 
 	if (success)
 	{	/* empty notification reply */
-		message = simaka_message_create(FALSE, in->get_identifier(in), EAP_AKA,
+		message = simaka_message_create(FALSE, this->identifier, EAP_AKA,
 										AKA_NOTIFICATION, this->crypto);
 		*out = message->generate(message, chunk_empty);
 		message->destroy(message);
 	}
 	else
 	{
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 	}
 	return NEED_MORE;
 }
 
 
-/**
- * Implementation of eap_method_t.process
- */
-static status_t process(private_eap_aka_peer_t *this,
-						eap_payload_t *in, eap_payload_t **out)
+METHOD(eap_method_t, process, status_t,
+	private_eap_aka_peer_t *this, eap_payload_t *in, eap_payload_t **out)
 {
 	simaka_message_t *message;
 	status_t status;
 
+	/* store received EAP message identifier */
+	this->identifier = in->get_identifier(in);
+
 	message = simaka_message_create_from_payload(in, this->crypto);
 	if (!message)
 	{
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 	if (!message->parse(message))
 	{
 		message->destroy(message);
-		*out = create_client_error(this, in->get_identifier(in));
+		*out = create_client_error(this);
 		return NEED_MORE;
 	}
 	switch (message->get_subtype(message))
@@ -491,7 +495,7 @@ static status_t process(private_eap_aka_peer_t *this,
 		default:
 			DBG1(DBG_IKE, "unable to process EAP-AKA subtype %N",
 				 simaka_subtype_names, message->get_subtype(message));
-			*out = create_client_error(this, in->get_identifier(in));
+			*out = create_client_error(this);
 			status = NEED_MORE;
 			break;
 	}
@@ -499,28 +503,22 @@ static status_t process(private_eap_aka_peer_t *this,
 	return status;
 }
 
-/**
- * Implementation of eap_method_t.initiate
- */
-static status_t initiate(private_eap_aka_peer_t *this, eap_payload_t **out)
+METHOD(eap_method_t, initiate, status_t,
+	private_eap_aka_peer_t *this, eap_payload_t **out)
 {
 	/* peer never initiates */
 	return FAILED;
 }
 
-/**
- * Implementation of eap_method_t.get_type.
- */
-static eap_type_t get_type(private_eap_aka_peer_t *this, u_int32_t *vendor)
+METHOD(eap_method_t, get_type, eap_type_t,
+	private_eap_aka_peer_t *this, u_int32_t *vendor)
 {
 	*vendor = 0;
 	return EAP_AKA;
 }
 
-/**
- * Implementation of eap_method_t.get_msk.
- */
-static status_t get_msk(private_eap_aka_peer_t *this, chunk_t *msk)
+METHOD(eap_method_t, get_msk, status_t,
+	private_eap_aka_peer_t *this, chunk_t *msk)
 {
 	if (this->msk.ptr)
 	{
@@ -530,18 +528,26 @@ static status_t get_msk(private_eap_aka_peer_t *this, chunk_t *msk)
 	return FAILED;
 }
 
-/**
- * Implementation of eap_method_t.is_mutual.
- */
-static bool is_mutual(private_eap_aka_peer_t *this)
+METHOD(eap_method_t, get_identifier, u_int8_t,
+	private_eap_aka_peer_t *this)
+{
+	return this->identifier;
+}
+
+METHOD(eap_method_t, set_identifier, void,
+	private_eap_aka_peer_t *this, u_int8_t identifier)
+{
+	this->identifier = identifier;
+}
+
+METHOD(eap_method_t, is_mutual, bool,
+	private_eap_aka_peer_t *this)
 {
 	return TRUE;
 }
 
-/**
- * Implementation of eap_method_t.destroy.
- */
-static void destroy(private_eap_aka_peer_t *this)
+METHOD(eap_method_t, destroy, void,
+	private_eap_aka_peer_t *this)
 {
 	this->crypto->destroy(this->crypto);
 	this->permanent->destroy(this->permanent);
@@ -557,25 +563,31 @@ static void destroy(private_eap_aka_peer_t *this)
 eap_aka_peer_t *eap_aka_peer_create(identification_t *server,
 									identification_t *peer)
 {
-	private_eap_aka_peer_t *this = malloc_thing(private_eap_aka_peer_t);
+	private_eap_aka_peer_t *this;
 
-	this->public.interface.initiate = (status_t(*)(eap_method_t*,eap_payload_t**))initiate;
-	this->public.interface.process = (status_t(*)(eap_method_t*,eap_payload_t*,eap_payload_t**))process;
-	this->public.interface.get_type = (eap_type_t(*)(eap_method_t*,u_int32_t*))get_type;
-	this->public.interface.is_mutual = (bool(*)(eap_method_t*))is_mutual;
-	this->public.interface.get_msk = (status_t(*)(eap_method_t*,chunk_t*))get_msk;
-	this->public.interface.destroy = (void(*)(eap_method_t*))destroy;
+	INIT(this,
+		.public = {
+			.interface = {
+				.initiate = _initiate,
+				.process = _process,
+				.get_type = _get_type,
+				.is_mutual = _is_mutual,
+				.get_msk = _get_msk,
+				.get_identifier = _get_identifier,
+				.set_identifier = _set_identifier,
+				.destroy = _destroy,
+			},
+		},
+		.crypto = simaka_crypto_create(),
+	);
 
-	this->crypto = simaka_crypto_create();
 	if (!this->crypto)
 	{
 		free(this);
 		return NULL;
 	}
+
 	this->permanent = peer->clone(peer);
-	this->pseudonym = NULL;
-	this->reauth = NULL;
-	this->msk = chunk_empty;
 
 	return &this->public;
 }
