@@ -19,6 +19,11 @@
 
 #include <daemon.h>
 
+METHOD(plugin_t, get_name, char*,
+	eap_tls_plugin_t *this)
+{
+	return "eap-tls";
+}
 
 METHOD(plugin_t, destroy, void,
 	eap_tls_plugin_t *this)
@@ -39,6 +44,8 @@ plugin_t *eap_tls_plugin_create()
 
 	INIT(this,
 		.plugin = {
+			.get_name = _get_name,
+			.reload = (void*)return_false,
 			.destroy = _destroy,
 		},
 	);
