@@ -32,9 +32,14 @@ struct private_socket_raw_plugin_t {
 	 * Implements plugin interface
 	 */
 	socket_raw_plugin_t public;
-
 };
 
+METHOD(plugin_t, get_name, char*,
+	private_socket_raw_plugin_t *this)
+{
+	return "socket-raw";
+}
+
 METHOD(plugin_t, destroy, void,
 	private_socket_raw_plugin_t *this)
 {
@@ -53,6 +58,8 @@ plugin_t *socket_raw_plugin_create()
 	INIT(this,
 		.public = {
 			.plugin = {
+				.get_name = _get_name,
+				.reload = (void*)return_false,
 				.destroy = _destroy,
 			},
 		},
