@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: context.h 3489 2008-02-22 09:40:58Z martin $
  */
 
 /**
@@ -34,11 +32,11 @@ typedef context_t *(*context_constructor_t)(void *param);
  * User specific session context, to extend.
  */
 struct context_t {
-	
+
 	/**
 	 * Destroy the context_t.
 	 */
 	void (*destroy) (context_t *this);
 };
 
-#endif /* CONTEXT_H_ @}*/
+#endif /** CONTEXT_H_ @}*/
