@@ -120,7 +120,7 @@ static char *getenv_cb(void *null, const char *key)
 	private_request_t *this = (private_request_t*)thread_this->get(thread_this);
 
 	value = FCGX_GetParam(key, this->req.envp);
-	return value ? strdup(value) : NULL;
+	return strdupnull(value);
 }
 
 /**
@@ -203,6 +203,14 @@ static char* get_query_data(private_request_t *this, char *name)
 	return hdf_get_valuef(this->hdf, "Query.%s", name);
 }
 
+/**
+ * Implementation of request_t.get_env_var.
+ */
+static char* get_env_var(private_request_t *this, char *name)
+{
+	return FCGX_GetParam(name, this->req.envp);
+}
+
 /**
  * Implementation of request_t.read_data.
  */
@@ -415,6 +423,7 @@ request_t *request_create(int fd, bool debug)
 	this->public.add_cookie = (void(*)(request_t*, char *name, char *value))add_cookie;
 	this->public.get_cookie = (char*(*)(request_t*,char*))get_cookie;
 	this->public.get_query_data = (char*(*)(request_t*, char *name))get_query_data;
+	this->public.get_env_var = (char*(*)(request_t*, char *name))get_env_var;
 	this->public.read_data = (int(*)(request_t*, char*, int))read_data;
 	this->public.session_closed = (bool(*)(request_t*))session_closed;
 	this->public.close_session = (void(*)(request_t*))close_session;
