@@ -40,9 +40,14 @@ struct private_attr_sql_plugin_t {
 	 * configuration attributes
 	 */
 	sql_attribute_t *attribute;
-
 };
 
+METHOD(plugin_t, get_name, char*,
+	private_attr_sql_plugin_t *this)
+{
+	return "attr-sql";
+}
+
 METHOD(plugin_t, destroy, void,
 	private_attr_sql_plugin_t *this)
 {
@@ -71,6 +76,8 @@ plugin_t *attr_sql_plugin_create()
 	INIT(this,
 		.public = {
 			.plugin = {
+				.get_name = _get_name,
+				.reload = (void*)return_false,
 				.destroy = _destroy,
 			},
 		},
