@@ -46,7 +46,9 @@ af_alg_signer_t *af_alg_signer_create(integrity_algorithm_t algo);
 
 /**
  * Probe algorithms and register af_alg_signer_create().
+ *
+ * @param plugin		plugin name to register algorithms for
  */
-void af_alg_signer_probe();
+void af_alg_signer_probe(char *plugin);
 
 #endif /** AF_ALG_SIGNER_H_ @}*/
