@@ -474,10 +474,8 @@ static bool mysql_enumerator_enumerate(mysql_enumerator_t *this, ...)
 	return TRUE;
 }
 
-/**
- * Implementation of database_t.query.
- */
-static enumerator_t* query(private_mysql_database_t *this, char *sql, ...)
+METHOD(database_t, query, enumerator_t*,
+	private_mysql_database_t *this, char *sql, ...)
 {
 	MYSQL_STMT *stmt;
 	va_list args;
@@ -563,10 +561,8 @@ static enumerator_t* query(private_mysql_database_t *this, char *sql, ...)
 	return (enumerator_t*)enumerator;
 }
 
-/**
- * Implementation of database_t.execute.
- */
-static int execute(private_mysql_database_t *this, int *rowid, char *sql, ...)
+METHOD(database_t, execute, int,
+	private_mysql_database_t *this, int *rowid, char *sql, ...)
 {
 	MYSQL_STMT *stmt;
 	va_list args;
@@ -594,18 +590,14 @@ static int execute(private_mysql_database_t *this, int *rowid, char *sql, ...)
 	return affected;
 }
 
-/**
- * Implementation of database_t.get_driver
- */
-static db_driver_t get_driver(private_mysql_database_t *this)
+METHOD(database_t, get_driver,db_driver_t,
+	private_mysql_database_t *this)
 {
 	return DB_MYSQL;
 }
 
-/**
- * Implementation of database_t.destroy
- */
-static void destroy(private_mysql_database_t *this)
+METHOD(database_t, destroy, void,
+	private_mysql_database_t *this)
 {
 	this->pool->destroy_function(this->pool, (void*)conn_destroy);
 	this->mutex->destroy(this->mutex);
@@ -677,12 +669,16 @@ mysql_database_t *mysql_database_create(char *uri)
 		return NULL;
 	}
 
-	this = malloc_thing(private_mysql_database_t);
-
-	this->public.db.query = (enumerator_t* (*)(database_t *this, char *sql, ...))query;
-	this->public.db.execute = (int (*)(database_t *this, int *rowid, char *sql, ...))execute;
-	this->public.db.get_driver = (db_driver_t(*)(database_t*))get_driver;
-	this->public.db.destroy = (void(*)(database_t*))destroy;
+	INIT(this,
+		.public = {
+			.db = {
+				.query = _query,
+				.execute = _execute,
+				.get_driver = _get_driver,
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	if (!parse_uri(this, uri))
 	{
