@@ -33,10 +33,14 @@ struct private_pem_plugin_t {
 	pem_plugin_t public;
 };
 
-/**
- * Implementation of pem_plugin_t.pemtroy
- */
-static void destroy(private_pem_plugin_t *this)
+METHOD(plugin_t, get_name, char*,
+	private_pem_plugin_t *this)
+{
+	return "pem";
+}
+
+METHOD(plugin_t, destroy, void,
+	private_pem_plugin_t *this)
 {
 	lib->creds->remove_builder(lib->creds,
 							   (builder_function_t)pem_private_key_load);
@@ -52,9 +56,17 @@ static void destroy(private_pem_plugin_t *this)
  */
 plugin_t *pem_plugin_create()
 {
-	private_pem_plugin_t *this = malloc_thing(private_pem_plugin_t);
+	private_pem_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
+	INIT(this,
+		.public = {
+			.plugin = {
+				.get_name = _get_name,
+				.reload = (void*)return_false,
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	/* register private key PEM decoding builders */
 	lib->creds->add_builder(lib->creds, CRED_PRIVATE_KEY, KEY_ANY, FALSE,
