@@ -32,10 +32,14 @@ struct private_pkcs1_plugin_t {
 	pkcs1_plugin_t public;
 };
 
-/**
- * Implementation of pkcs1_plugin_t.pkcs1troy
- */
-static void destroy(private_pkcs1_plugin_t *this)
+METHOD(plugin_t, get_name, char*,
+	private_pkcs1_plugin_t *this)
+{
+	return "pkcs1";
+}
+
+METHOD(plugin_t, destroy, void,
+	private_pkcs1_plugin_t *this)
 {
 	lib->creds->remove_builder(lib->creds,
 							(builder_function_t)pkcs1_public_key_load);
@@ -52,9 +56,17 @@ static void destroy(private_pkcs1_plugin_t *this)
  */
 plugin_t *pkcs1_plugin_create()
 {
-	private_pkcs1_plugin_t *this = malloc_thing(private_pkcs1_plugin_t);
+	private_pkcs1_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
+	INIT(this,
+		.public = {
+			.plugin = {
+				.get_name = _get_name,
+				.reload = (void*)return_false,
+				.destroy = _destroy,
+			},
+		},
+	);
 
 	lib->creds->add_builder(lib->creds, CRED_PUBLIC_KEY, KEY_ANY, FALSE,
 							(builder_function_t)pkcs1_public_key_load);
