@@ -32,10 +32,14 @@ struct private_sha1_plugin_t {
 	sha1_plugin_t public;
 };
 
-/**
- * Implementation of sha1_plugin_t.destroy
- */
-static void destroy(private_sha1_plugin_t *this)
+METHOD(plugin_t, get_name, char*,
+	private_sha1_plugin_t *this)
+{
+	return "sha1";
+}
+
+METHOD(plugin_t, destroy, void,
+	private_sha1_plugin_t *this)
 {
 	lib->crypto->remove_hasher(lib->crypto,
 							   (hasher_constructor_t)sha1_hasher_create);
@@ -49,13 +53,21 @@ static void destroy(private_sha1_plugin_t *this)
  */
 plugin_t *sha1_plugin_create()
 {
-	private_sha1_plugin_t *this = malloc_thing(private_sha1_plugin_t);
+	private_sha1_plugin_t *this;
 
-	this->public.plugin.destroy = (void(*)(plugin_t*))destroy;
+	INIT(this,
+		.public = {
+			.plugin = {
+				.get_name = _get_name,
+				.reload = (void*)return_false,
+				.destroy = _destroy,
+			},
+		},
+	);
 
-	lib->crypto->add_hasher(lib->crypto, HASH_SHA1,
+	lib->crypto->add_hasher(lib->crypto, HASH_SHA1, get_name(this),
 							(hasher_constructor_t)sha1_hasher_create);
-	lib->crypto->add_prf(lib->crypto, PRF_KEYED_SHA1,
+	lib->crypto->add_prf(lib->crypto, PRF_KEYED_SHA1, get_name(this),
 							(prf_constructor_t)sha1_prf_create);
 
 	return &this->public.plugin;
