@@ -31,7 +31,7 @@ struct private_sha512_hasher_t {
 	 * Public interface for this hasher.
 	 */
 	sha2_hasher_t public;
-	
+
 	unsigned char   sha_out[128];   /* results are here, bytes 0..47/0..63 */
 	u_int64_t       sha_H[8];
 	u_int64_t       sha_blocks;
@@ -50,7 +50,7 @@ struct private_sha256_hasher_t {
 	 * Public interface for this hasher.
 	 */
 	sha2_hasher_t public;
-	
+
 	unsigned char   sha_out[64];    /* results are here, bytes 0...31 */
 	u_int32_t       sha_H[8];
 	u_int64_t       sha_blocks;
@@ -58,6 +58,11 @@ struct private_sha256_hasher_t {
 };
 
 
+static const u_int32_t sha224_hashInit[8] = {
+	0xc1059ed8, 0x367cd507, 0x3070dd17, 0xf70e5939, 0xffc00b31, 0x68581511,
+	0x64f98fa7, 0xbefa4fa4
+};
+
 static const u_int32_t sha256_hashInit[8] = {
 	0x6a09e667, 0xbb67ae85, 0x3c6ef372, 0xa54ff53a, 0x510e527f, 0x9b05688c,
 	0x1f83d9ab, 0x5be0cd19
@@ -134,7 +139,7 @@ static const u_int64_t sha512_K[80] = {
 /**
  * Single block SHA256 transformation
  */
-static void sha256_transform(private_sha256_hasher_t *ctx, 
+static void sha256_transform(private_sha256_hasher_t *ctx,
 							 const unsigned char *datap)
 {
 	register int    j;
@@ -163,7 +168,7 @@ static void sha256_transform(private_sha256_hasher_t *ctx,
 	j = 0;
 	do
 	{
-		if(j >= 16) 
+		if(j >= 16)
 		{
 			Wm2 = W[j - 2];
 			Wm15 = W[j - 15];
@@ -193,7 +198,7 @@ static void sha256_transform(private_sha256_hasher_t *ctx,
 /**
  * Update SHA256 hash
  */
-static void sha256_write(private_sha256_hasher_t *ctx, 
+static void sha256_write(private_sha256_hasher_t *ctx,
 						 const unsigned char *datap, int length)
 {
 	while(length > 0)
@@ -238,7 +243,7 @@ static void sha256_final(private_sha256_hasher_t *ctx)
 	{
 		sha256_write(ctx, &padByte, 1);
 	}
-	
+
 	/* write bit length, big endian byte order */
 	ctx->sha_out[56] = bitLength >> 56;
 	ctx->sha_out[57] = bitLength >> 48;
@@ -249,7 +254,7 @@ static void sha256_final(private_sha256_hasher_t *ctx)
 	ctx->sha_out[62] = bitLength >> 8;
 	ctx->sha_out[63] = bitLength;
 	sha256_transform(ctx, &ctx->sha_out[0]);
-	
+
 	/* return results in ctx->sha_out[0...31] */
 	datap = &ctx->sha_out[0];
 	j = 0;
@@ -278,7 +283,7 @@ static void sha256_final(private_sha256_hasher_t *ctx)
 /**
  * Single block SHA384/SHA512 transformation
  */
-static void sha512_transform(private_sha512_hasher_t *ctx, 
+static void sha512_transform(private_sha512_hasher_t *ctx,
 							 const unsigned char *datap)
 {
 	register int    j;
@@ -338,14 +343,14 @@ static void sha512_transform(private_sha512_hasher_t *ctx,
 /**
  * Update a SHA384/SHA512 hash
  */
-static void sha512_write(private_sha512_hasher_t *ctx, 
+static void sha512_write(private_sha512_hasher_t *ctx,
 						 const unsigned char *datap, int length)
 {
-	while(length > 0) 
+	while(length > 0)
 	{
-		if(!ctx->sha_bufCnt) 
+		if(!ctx->sha_bufCnt)
 		{
-			while(length >= sizeof(ctx->sha_out)) 
+			while(length >= sizeof(ctx->sha_out))
 			{
 				sha512_transform(ctx, datap);
 				datap += sizeof(ctx->sha_out);
@@ -355,7 +360,7 @@ static void sha512_write(private_sha512_hasher_t *ctx,
 		}
 		ctx->sha_out[ctx->sha_bufCnt] = *datap++;
 		length--;
-		if(++ctx->sha_bufCnt == sizeof(ctx->sha_out)) 
+		if(++ctx->sha_bufCnt == sizeof(ctx->sha_out))
 		{
 			sha512_transform(ctx, &ctx->sha_out[0]);
 			ctx->sha_bufCnt = 0;
@@ -380,7 +385,7 @@ static void sha512_final(private_sha512_hasher_t *ctx)
 
 	/* pad extra space with zeroes */
 	padByte = 0;
-	while(ctx->sha_bufCnt != 112) 
+	while(ctx->sha_bufCnt != 112)
 	{
 		sha512_write(ctx, &padByte, 1);
 	}
@@ -403,7 +408,7 @@ static void sha512_final(private_sha512_hasher_t *ctx)
 	ctx->sha_out[126] = bitLength >> 8;
 	ctx->sha_out[127] = bitLength;
 	sha512_transform(ctx, &ctx->sha_out[0]);
-	
+
 	/* return results in ctx->sha_out[0...63] */
 	datap = &ctx->sha_out[0];
 	j = 0;
@@ -421,10 +426,25 @@ static void sha512_final(private_sha512_hasher_t *ctx)
 	} while(++j < 8);
 }
 
+/**
+ * Implementation of hasher_t.get_hash for SHA224.
+ */
+static void get_hash224(private_sha256_hasher_t *this,
+						chunk_t chunk, u_int8_t *buffer)
+{
+	sha256_write(this, chunk.ptr, chunk.len);
+	if (buffer != NULL)
+	{
+		sha256_final(this);
+		memcpy(buffer, this->sha_out, HASH_SIZE_SHA224);
+		this->public.hasher_interface.reset(&(this->public.hasher_interface));
+	}
+}
+
 /**
  * Implementation of hasher_t.get_hash for SHA256.
  */
-static void get_hash256(private_sha256_hasher_t *this, 
+static void get_hash256(private_sha256_hasher_t *this,
 						chunk_t chunk, u_int8_t *buffer)
 {
 	sha256_write(this, chunk.ptr, chunk.len);
@@ -467,13 +487,32 @@ static void get_hash512(private_sha512_hasher_t *this,
 }
 
 /**
- * Implementation of hasher_t.allocate_hash for SHA256.
+ * Implementation of hasher_t.allocate_hash for SHA224.
  */
-static void allocate_hash256(private_sha256_hasher_t *this, 
+static void allocate_hash224(private_sha256_hasher_t *this,
 							 chunk_t chunk, chunk_t *hash)
 {
 	chunk_t allocated_hash;
-	
+
+	sha256_write(this, chunk.ptr, chunk.len);
+	if (hash != NULL)
+	{
+		sha256_final(this);
+		allocated_hash = chunk_alloc(HASH_SIZE_SHA224);
+		memcpy(allocated_hash.ptr, this->sha_out, HASH_SIZE_SHA224);
+		this->public.hasher_interface.reset(&(this->public.hasher_interface));
+		*hash = allocated_hash;
+	}
+}
+
+/**
+ * Implementation of hasher_t.allocate_hash for SHA256.
+ */
+static void allocate_hash256(private_sha256_hasher_t *this,
+							 chunk_t chunk, chunk_t *hash)
+{
+	chunk_t allocated_hash;
+
 	sha256_write(this, chunk.ptr, chunk.len);
 	if (hash != NULL)
 	{
@@ -488,11 +527,11 @@ static void allocate_hash256(private_sha256_hasher_t *this,
 /**
  * Implementation of hasher_t.allocate_hash for SHA384.
  */
-static void allocate_hash384(private_sha512_hasher_t *this, 
+static void allocate_hash384(private_sha512_hasher_t *this,
 							 chunk_t chunk, chunk_t *hash)
 {
 	chunk_t allocated_hash;
-	
+
 	sha512_write(this, chunk.ptr, chunk.len);
 	if (hash != NULL)
 	{
@@ -507,11 +546,11 @@ static void allocate_hash384(private_sha512_hasher_t *this,
 /**
  * Implementation of hasher_t.allocate_hash for SHA512.
  */
-static void allocate_hash512(private_sha512_hasher_t *this, 
+static void allocate_hash512(private_sha512_hasher_t *this,
 							 chunk_t chunk, chunk_t *hash)
 {
 	chunk_t allocated_hash;
-	
+
 	sha512_write(this, chunk.ptr, chunk.len);
 	if (hash != NULL)
 	{
@@ -523,6 +562,14 @@ static void allocate_hash512(private_sha512_hasher_t *this,
 	}
 }
 
+/**
+ * Implementation of hasher_t.get_hash_size for SHA224.
+ */
+static size_t get_hash_size224(private_sha256_hasher_t *this)
+{
+	return HASH_SIZE_SHA224;
+}
+
 /**
  * Implementation of hasher_t.get_hash_size for SHA256.
  */
@@ -530,7 +577,7 @@ static size_t get_hash_size256(private_sha256_hasher_t *this)
 {
 	return HASH_SIZE_SHA256;
 }
-	
+
 /**
  * Implementation of hasher_t.get_hash_size for SHA384.
  */
@@ -538,7 +585,7 @@ static size_t get_hash_size384(private_sha512_hasher_t *this)
 {
 	return HASH_SIZE_SHA384;
 }
-	
+
 /**
  * Implementation of hasher_t.get_hash_size for SHA512.
  */
@@ -547,6 +594,16 @@ static size_t get_hash_size512(private_sha512_hasher_t *this)
 	return HASH_SIZE_SHA512;
 }
 
+/**
+ * Implementation of hasher_t.reset for SHA224
+ */
+static void reset224(private_sha256_hasher_t *ctx)
+{
+	memcpy(&ctx->sha_H[0], &sha224_hashInit[0], sizeof(ctx->sha_H));
+	ctx->sha_blocks = 0;
+	ctx->sha_bufCnt = 0;
+}
+
 /**
  * Implementation of hasher_t.reset for SHA256
  */
@@ -593,9 +650,16 @@ static void destroy(sha2_hasher_t *this)
 sha2_hasher_t *sha2_hasher_create(hash_algorithm_t algorithm)
 {
 	sha2_hasher_t *this;
-	
+
 	switch (algorithm)
 	{
+		case HASH_SHA224:
+			this = (sha2_hasher_t*)malloc_thing(private_sha256_hasher_t);
+			this->hasher_interface.reset = (void(*)(hasher_t*))reset224;
+			this->hasher_interface.get_hash_size = (size_t(*)(hasher_t*))get_hash_size224;
+			this->hasher_interface.get_hash = (void(*)(hasher_t*,chunk_t,u_int8_t*))get_hash224;
+			this->hasher_interface.allocate_hash = (void(*)(hasher_t*,chunk_t,chunk_t*))allocate_hash224;
+			break;
 		case HASH_SHA256:
 			this = (sha2_hasher_t*)malloc_thing(private_sha256_hasher_t);
 			this->hasher_interface.reset = (void(*)(hasher_t*))reset256;
@@ -622,9 +686,9 @@ sha2_hasher_t *sha2_hasher_create(hash_algorithm_t algorithm)
 			return NULL;
 	}
 	this->hasher_interface.destroy = (void(*)(hasher_t*))destroy;
-	
+
 	/* initialize */
 	this->hasher_interface.reset(&this->hasher_interface);
-	
+
 	return this;
 }
