@@ -99,6 +99,14 @@ void memxor(u_int8_t dst[], u_int8_t src[], size_t n)
 	}
 }
 
+/**
+ * Described in header.
+ */
+void memwipe_noinline(void *ptr, size_t n)
+{
+	memwipe_inline(ptr, n);
+}
+
 /**
  * Described in header.
  */
@@ -246,6 +254,14 @@ bool return_false()
 	return FALSE;
 }
 
+/**
+ * returns FAILED
+ */
+status_t return_failed()
+{
+	return FAILED;
+}
+
 /**
  * nop operation
  */
@@ -279,7 +295,7 @@ bool ref_put(refcount_t *ref)
 	bool more_refs;
 
 	pthread_mutex_lock(&ref_mutex);
-	more_refs = --(*ref);
+	more_refs = --(*ref) > 0;
 	pthread_mutex_unlock(&ref_mutex);
 	return !more_refs;
 }
