@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: control_controller.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -44,4 +42,4 @@ struct control_controller_t {
  */
 controller_t *control_controller_create(context_t *context, void *param);
 
-#endif /* CONTROL_CONTROLLER_H_ */
+#endif /** CONTROL_CONTROLLER_H_ @}*/
