@@ -12,9 +12,10 @@
  * for more details.
  */
 
-extern int
-	adns_qfd,   /* file descriptor for sending queries to adns */
-	adns_afd;   /* file descriptor for receiving answers from adns */
+#include <utils/identification.h>
+
+extern int adns_qfd;   /* file descriptor for sending queries to adns */
+extern int adns_afd;   /* file descriptor for receiving answers from adns */
 extern const char *pluto_adns_option;   /* path from --pluto_adns */
 extern void init_adns(void);
 extern void stop_adns(void);
@@ -33,13 +34,13 @@ struct adns_continuation;       /* forward declaration (not far!) */
 typedef void (*cont_fn_t)(struct adns_continuation *cr, err_t ugh);
 
 struct adns_continuation {
-	unsigned long qtid; /* query transaction id number */
-	int type;   /* T_TXT or T_KEY, selecting rr type of interest */
-	cont_fn_t cont_fn;  /* function to carry on suspended work */
-	struct id id;       /* subject of query */
+	unsigned long qtid;    /* query transaction id number */
+	int type;              /* T_TXT or T_KEY, selecting rr type of interest */
+	cont_fn_t cont_fn;     /* function to carry on suspended work */
+	identification_t *id;  /* subject of query */
 	bool sgw_specified;
-	struct id sgw_id;   /* peer, if constrained */
-	lset_t debugging;   /* only used #ifdef DEBUG, but don't want layout to change */
+	identification_t *sgw_id; /* peer, if constrained */
+	lset_t debugging;      /* only used #ifdef DEBUG, but don't want layout to change */
 	struct gw_info *gateways_from_dns;  /* answer, if looking for our TXT rrs */
 #ifdef USE_KEYRR
 	struct pubkey_list *keys_from_dns;  /* answer, if looking for KEY rrs */
@@ -47,7 +48,7 @@ struct adns_continuation {
 	struct adns_continuation *previous, *next;
 	struct pubkey *last_info;  /* the last structure we accumulated */
 #ifdef USE_LWRES
-	bool used;  /* have we called the cont_fn yet? */
+	bool used;             /* have we called the cont_fn yet? */
 	struct {
 		u_char name_buf[NS_MAXDNAME + 2];
 	} query;
@@ -56,8 +57,8 @@ struct adns_continuation {
 #endif /* ! USE_LWRES */
 };
 
-extern err_t start_adns_query(const struct id *id       /* domain to query */
-	, const struct id *sgw_id   /* if non-null, any accepted gw_info must match */
+extern err_t start_adns_query(identification_t *id       /* domain to query */
+	, identification_t *sgw_id   /* if non-null, any accepted gw_info must match */
 	, int type  /* T_TXT or T_KEY, selecting rr type of interest */
 	, cont_fn_t cont_fn /* continuation function */
 	, struct adns_continuation *cr);
@@ -65,18 +66,17 @@ extern err_t start_adns_query(const struct id *id       /* domain to query */
 
 /* Gateway info gleaned from reverse DNS of client */
 struct gw_info {
-	unsigned refcnt;    /* reference counted! */
-	unsigned pref;      /* preference: lower is better */
-#define NO_TIME ((time_t) -2)   /* time_t value meaning "not_yet" */
-	struct id client_id;        /* id of client of peer */
-	struct id gw_id;    /* id of peer (if id_is_ipaddr, .ip_addr is address) */
+	unsigned refcnt;             /* reference counted! */
+	unsigned pref;               /* preference: lower is better */
+#define NO_TIME ((time_t) -2)    /* time_t value meaning "not_yet" */
+	identification_t* client_id; /* id of client of peer */
+	identification_t* gw_id;     /* id of peer (if id_is_ipaddr, .ip_addr is address) */
 	bool gw_key_present;
 	struct pubkey *key;
 	struct gw_info *next;
 };
 
-extern void gw_addref(struct gw_info *gw)
-	, gw_delref(struct gw_info **gwp);
-
+extern void gw_addref(struct gw_info *gw);
+extern void gw_delref(struct gw_info **gwp);
 extern void reset_adns_restart_count(void);
 
