@@ -12,6 +12,11 @@
  * for more details.
  */
 
+#ifndef _IPSEC_DOI_H
+#define _IPSEC_DOI_H
+
+#include "defs.h"
+
 extern void echo_hdr(struct msg_digest *md, bool enc, u_int8_t np);
 
 extern void ipsecdoi_initiate(int whack_sock, struct connection *c
@@ -95,8 +100,9 @@ extern void dpd_timeout(struct state *st);
 			DBG_cond_dump(DBG_CRYPT, "received " hash_name ":", hash_pbs->cur, pbs_left(hash_pbs)); \
 			loglog(RC_LOG_SERIOUS, "received " hash_name " does not match computed value in " msg_name); \
 			/* XXX Could send notification back */ \
-			return STF_FAIL + INVALID_HASH_INFORMATION; \
+			return STF_FAIL + ISAKMP_INVALID_HASH_INFORMATION; \
 		} \
 	}
 
+#endif /* _IPSEC_DOI_H */
 
