@@ -1185,7 +1185,7 @@ static bool setup_half_ipsec_sa(struct state *st, bool inbound)
 						host_dst, ipcomp_spi, said_next->proto, c->spd.reqid,
 						mark, 0, &lt_none, ENCR_UNDEFINED, chunk_empty,
 						AUTH_UNDEFINED, chunk_empty, mode,
-						st->st_ipcomp.attrs.transid, 0 /* cpi */, FALSE,
+						st->st_ipcomp.attrs.transid, 0 /* cpi */, FALSE, FALSE,
 						inbound, NULL, NULL) != SUCCESS)
 		{
 			goto fail;
@@ -1294,7 +1294,7 @@ static bool setup_half_ipsec_sa(struct state *st, bool inbound)
 						host_dst, esp_spi, said_next->proto, c->spd.reqid,
 						mark, 0, &lt_none, enc_alg, enc_key,
 						auth_alg, auth_key, mode, IPCOMP_NONE, 0 /* cpi */,
-						encap, inbound, NULL, NULL) != SUCCESS)
+						encap, FALSE, inbound, NULL, NULL) != SUCCESS)
 		{
 			goto fail;
 		}
@@ -1327,7 +1327,7 @@ static bool setup_half_ipsec_sa(struct state *st, bool inbound)
 						host_dst, ah_spi, said_next->proto, c->spd.reqid,
 						mark, 0, &lt_none, ENCR_UNDEFINED, chunk_empty,
 						auth_alg, auth_key, mode, IPCOMP_NONE, 0 /* cpi */,
-						FALSE, inbound, NULL, NULL) != SUCCESS)
+						FALSE, FALSE, inbound, NULL, NULL) != SUCCESS)
 		{
 			goto fail;
 		}
