@@ -337,16 +337,16 @@ open_peerlog(connection_t *c)
 
 		/* copy IP address, turning : and . into / */
 		{
-			char c, *p, *q;
+			char ch, *p, *q;
 
 			p = peername;
 			q = dname;
 			do {
-				c = *p++;
-				if (c == '.' || c == ':')
-					c = '/';
-				*q++ = c;
-			} while (c != '\0');
+				ch = *p++;
+				if (ch == '.' || ch == ':')
+					ch = '/';
+				*q++ = ch;
+			} while (ch != '\0');
 		}
 
 		lf_len = peernamelen * 2
@@ -834,7 +834,8 @@ DBG_dump(const char *label, const void *p, size_t len)
 
 static void show_loaded_plugins()
 {
-	char buf[BUF_LEN], *plugin;
+	char buf[BUF_LEN];
+	plugin_t *plugin;
 	int len = 0;
 	enumerator_t *enumerator;
 
@@ -842,7 +843,7 @@ static void show_loaded_plugins()
 	enumerator = lib->plugins->create_plugin_enumerator(lib->plugins);
 	while (len < BUF_LEN && enumerator->enumerate(enumerator, &plugin))
 	{
-		len += snprintf(&buf[len], BUF_LEN-len, "%s ", plugin);
+		len += snprintf(&buf[len], BUF_LEN-len, "%s ", plugin->get_name(plugin));
 	}
 	enumerator->destroy(enumerator);
 	whack_log(RC_COMMENT, "loaded plugins: %s", buf);
