@@ -1,7 +1,7 @@
 /**
  * @file scep.h
  * @brief SCEP specific functions
- * 
+ *
  * Contains functions to build and parse SCEP requests and replies
  */
 
@@ -23,6 +23,8 @@
 #ifndef _SCEP_H
 #define _SCEP_H
 
+#include <credentials/certificates/certificate.h>
+
 #include "../pluto/defs.h"
 #include "../pluto/pkcs7.h"
 
@@ -81,13 +83,13 @@ extern chunk_t scep_transId_attribute(chunk_t transaction_id);
 extern chunk_t scep_messageType_attribute(scep_msg_t m);
 extern chunk_t scep_senderNonce_attribute(void);
 extern chunk_t scep_build_request(chunk_t data, chunk_t transID, scep_msg_t msg,
-								  const x509cert_t *enc_cert, int enc_alg,
-								  const x509cert_t *signer_cert, int digest_alg,
+								  certificate_t *enc_cert, int enc_alg,
+								  certificate_t *signer_cert, int digest_alg,
 								  private_key_t *private_key);
 extern bool scep_http_request(const char *url, chunk_t pkcs7, scep_op_t op,
 							  bool http_get_request, chunk_t *response);
 extern err_t scep_parse_response(chunk_t response, chunk_t transID,
 								 contentInfo_t *data, scep_attributes_t *attrs,
-								 x509cert_t *signer_cert);
+								 certificate_t *signer_cert);
 
 #endif /* _SCEP_H */
