@@ -279,14 +279,15 @@ usage(const char *message)
  */
 static void print_plugins()
 {
-	char buf[BUF_LEN], *plugin;
+	char buf[BUF_LEN];
+	plugin_t *plugin;
 	int len = 0;
 	enumerator_t *enumerator;
 
 	enumerator = lib->plugins->create_plugin_enumerator(lib->plugins);
 	while (len < BUF_LEN && enumerator->enumerate(enumerator, &plugin))
 	{
-		len += snprintf(&buf[len], BUF_LEN-len, "%s ", plugin);
+		len += snprintf(&buf[len], BUF_LEN-len, "%s ", plugin->get_name(plugin));
 	}
 	enumerator->destroy(enumerator);
 	DBG1(DBG_LIB, "  loaded plugins: %s", buf);
@@ -1019,6 +1020,7 @@ int main(int argc, char **argv)
 	 */
 	if (filetype_out & CERT)
 	{
+		bool stored = FALSE;
 		certificate_t *cert;
 		enumerator_t  *enumerator;
 		char *path = concatenate_paths(CA_CERT_PATH, file_in_cacert_sig);
@@ -1128,7 +1130,6 @@ int main(int argc, char **argv)
 		enumerator = certs->create_enumerator(certs);
 		while (enumerator->enumerate(enumerator, &cert))
 		{
-			bool stored = FALSE;
 			x509_t *x509 = (x509_t*)cert;
 
 			if (!(x509->get_flags(x509) & X509_CA))
