@@ -1,5 +1,5 @@
-/* C code produced by gperf version 3.0.2 */
-/* Command-line: /scratchbox/tools/bin/gperf -m 10 -D -C -G -t  */
+/* C code produced by gperf version 3.0.3 */
+/* Command-line: /usr/bin/gperf -m 10 -D -C -G -t  */
 /* Computed positions: -k'1,5,7' */
 
 #if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
@@ -174,6 +174,9 @@ static const short lookup[] =
 
 #ifdef __GNUC__
 __inline
+#ifdef __GNUC_STDC_INLINE__
+__attribute__ ((__gnu_inline__))
+#endif
 #endif
 const struct stroke_token *
 in_word_set (str, len)
